// Main chords generator (perfect major, perfect minor, dominant seventh and octaves)
// Copyright (C) 2015 - Paul Bignier

p = funcprot();
funcprot(0);

// Defining the GUI
fig = figure(...
    "figure_name", "Chords",...
    "figure_size", [350 200],...
    "layout", "gridbag",...
    "background", 8,...
    "menubar_visible", "off",...
    "toolbar_visible", "off",...
    "default_axes", "off",...
    "visible", "off",...
    "tag", "main_figure");

// Frame for the uicontrols
c = createConstraints("gridbag", [1 1 2 2], [1 1], "both", "right");
main_frame = uicontrol(...
    "style", "frame",...
    "constraints", c,...
    "layout", "gridbag",...
    "tag", "main_frame");

// Text area
c = createConstraints("gridbag", [1 1 1 1], [1/2 1/2], "both", "upper");
text = uicontrol(...
    "parent", main_frame,...
    "style", "text",...
    "string", " Fundamental note frequency (Hz): ",...
    "horizontalAlignment", "right",...
    "constraints", c,...
    "tag", "text");

// Edit box
c = createConstraints("gridbag", [2 1 1 1], [1/2 1/2], "both", "upper");
freqBox = uicontrol(...
    "parent", main_frame,...
    "style", "edit",...
    "string", "440",...
    "constraints", c,...
    "tag", "freqBox");

// Show chords button
c = createConstraints("gridbag", [1 2 2 1], [1 1/2], "both", "upper");
showButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "string", "Show main related chords",...
    "constraints", c,...
    "callback", "show",...
    "tag", "show");

function show()
    updateStatusBar(" Generating and plotting the chords...");

    eb = get("freqBox");
    set(eb, "enable", "off");
    str = get(eb, "string");

    // Check the input value
    err = execstr("f = evstr("+str+")", "errcatch");
    if err <> 0 then
        messagebox("Error: please enter a real scalar number", "Wrong inserted value", "error");
        return;
    end
    if type(f) <> 1  then
        messagebox("Error: please enter a real scalar number", "Wrong inserted value", "error");
        return;
    end
    if ~isscalar(f) | ~isreal(f) then
        messagebox("Error: please enter a real scalar number", "Wrong inserted value", "error");
        return;
    end

    // At first we create 0.5 seconds of sound parameters
    t = soundsec(0.5);
    twoPiT = 2*%pi*t;

    // Decide a first frequency for the first note (fundamental)
    note = sin(twoPiT*f);

    // Set the frame for the plots to come
    fig = figure(...
        "figure_size", [900 900],...
        "figure_name", "Main chords related to the "+string(f)+"-frequency note",...
        "background", 8);

    // Major chord
    subplot(4, 1, 1);

    // Add a major third and a perfect fifth
    // Major third (mediant)
    MTratio = 5/4;
    mediant = sin(twoPiT*f*MTratio);
    twoChord = note + mediant;
    // Perfect fifth (dominant)
    PFratio = 3/2;
    dominant = sin(twoPiT*f*PFratio);
    perfectMajor = twoChord + dominant;

    // Frequency plot of the generated sound (frequency vs volume)
    analyze(perfectMajor, 100, 1600);
    xtitle("Perfect major");

    // Play the sound
    playsnd(perfectMajor);


    // Minor chord
    subplot(4, 1, 2);

    // Add a minor third and a perfect fifth
    // Minor third (mediant)
    MTratio = 6/5;
    mediant = sin(twoPiT*f*MTratio);
    twoChord = note + mediant;
    // Perfect fifth (dominant)
    PFratio = 3/2;
    dominant = sin(twoPiT*f*PFratio);
    perfectMinor = twoChord + dominant;

    // Frequency plot of the generated sound (frequency vs volume)
    analyze(perfectMinor, 100, 1600);
    xtitle("Perfect minor");

    // Play the sound
    playsnd(perfectMinor);


    // Dominant seventh chord
    subplot(4, 1, 3);

    // Start from the dominant
    f_dom= f*3/2;
    fundamental = sin(twoPiT*f_dom);

    // Add a major third, a perfect fifth and a minor seventh
    // Major third (mediant)
    MTratio = 5/4;
    mediant = sin(twoPiT*f_dom*MTratio);
    twoChord = fundamental + mediant;
    // Perfect fifth (dominant)
    PFratio = 3/2;
    dominant = sin(twoPiT*f_dom*PFratio);
    perfectMajor = twoChord + dominant;
    // Minor Seventh (subtonic)
    MSratio = 9/5;
    seventh = sin(twoPiT*f_dom*MSratio);
    dominantSeventh = perfectMajor + seventh;

    // Frequency plot of the generated sound (frequency vs volume)
    analyze(dominantSeventh, 100, 1600);
    xtitle("Dominant seventh");

    // Play the sound
    playsnd(dominantSeventh);


    // Octaves
    subplot(4, 1, 4);

    // Generate the two octaves and the chord
    f_upper = f*2;
    f_lower = f/2;
    octaveU = sin(twoPiT*f_upper);
    octaveL = sin(twoPiT*f_lower);
    octaves = octaveL + note + octaveU;

    // Frequency plot of the generated sound (frequency vs volume)
    analyze(octaves, 100, 1600);
    xtitle("Surrounding octaves");

    // Play the sound
    playsnd(octaves);

    set(eb, "enable", "on");
    updateStatusBar("");
endfunction

function updateStatusBar(msg)
    set("main_figure", "info_message", msg);
endfunction

funcprot(p);

// Display the GUI
fig.visible = "on";
