// Filter a sound (low and high pass filters)
// Music: Erik Satie - Gymnopédie No 1
// Copyright (C) 2015-2017 - Paul Bignier

version = getversion("scilab");
if version(1,1) <= 5 then
    stacksize("max");
end

p = funcprot();
funcprot(0);

freqUpperLimit = 500; // iir() accepts frequencies up to 500Hz
freqLowerLimit = 1;

// Defining the GUI
fig = figure(...
    "figure_name", "Sound filtering (low and high pass filters)",...
    "figure_size", [800 500],...
    "layout", "gridbag",...
    "background", 8,...
    "menubar_visible", "off",...
    "toolbar_visible", "off",...
    "default_axes", "off",...
    "visible", "off",...
    "tag", "main_figure");

// Frame for the uicontrols
c = createConstraints("gridbag", [1 1 3 4], [1 1], "both", "right");
main_frame = uicontrol(...
    "style", "frame",...
    "constraints", c,...
    "layout", "gridbag",...
    "tag", "main_frame");

// Load file button
c = createConstraints("gridbag", [1 1 3 1], [1 1/4], "both", "upper");
loadButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "icon", "document-open",...
    "string", " Load a sound ",...
    "constraints", c,...
    "callback", "[piece, rate] = loadFile()",...
    "tag", "loadButton");

// Frame for the frequency editing
c = createConstraints("gridbag", [1 2 2 1], [2/3 1/4], "both", "right");
frame_edit = uicontrol(...
    "parent", main_frame,...
    "style", "frame",...
    "constraints", c,...
    "layout", "gridbag",...
    "tag", "frame_edit");

// Frequency text
c = createConstraints("gridbag", [1 1 1 3], [1/2 1], "both", "upper");
text = uicontrol(...
    "parent", frame_edit,...
    "style", "text",...
    "string", " Cutoff frequency (1-"+string(freqUpperLimit)+" Hz): ",...
    "horizontalAlignment", "center",...
    "constraints", c,...
    "tag", "freqText");

// Frame above the edit box
c = createConstraints("gridbag", [2 1 1 1], [1/2 1/3], "both", "left");
frame_empty1 = uicontrol(...
    "parent", frame_edit,...
    "style", "frame",...
    "constraints", c,...
    "layout", "gridbag",...
    "tag", "frame_empty1");

// Frequency edit box
c = createConstraints("gridbag", [2 2 1 1], [1/2 1/3], "both", "left");
freqBox = uicontrol(...
    "parent", frame_edit,...
    "style", "edit",...
    "string", "220",...
    "constraints", c,...
    "callback", "Filter = newFrequency()",...
    "tag", "freqBox");

// Frame underneath the edit box
c = createConstraints("gridbag", [2 3 1 1], [1/2 1/3], "both", "left");
frame_empty2 = uicontrol(...
    "parent", frame_edit,...
    "style", "frame",...
    "constraints", c,...
    "layout", "gridbag",...
    "tag", "frame_empty2");

// Frame for the filter radio buttons
c = createConstraints("gridbag", [3 2 1 1], [1/3 1/4], "both", "right");
frame_radio = uicontrol(...
    "parent", main_frame,...
    "style", "frame",...
    "constraints", c,...
    "layout", "gridbag",...
    "tag", "frame_radio");

// High-pass filter radio button
c = createConstraints("gridbag", [1 1 1 1], [1/2 1], "both", "upper");
highRadio = uicontrol(...
    "parent", frame_radio,...
    "style", "radiobutton",...
    "string", "High-pass",...
    "groupname", "filters",...
    "constraints", c,...
    "callback", "Filter = highPass()",...
    "tag", "highRadio");
highRadio.value = 1;

// Low-pass filter radio button
c = createConstraints("gridbag", [1 2 1 1], [1/2 1], "both", "upper");
lowRadio = uicontrol(...
    "parent", frame_radio,...
    "style", "radiobutton",...
    "string", "Low-pass",...
    "groupname", "filters",...
    "constraints", c,...
    "callback", "Filter = lowPass()",...
    "tag", "lowRadio");

// Apply and play button
c = createConstraints("gridbag", [1 3 1 1], [1/3 1/4], "both", "upper");
applyPlayButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "string", "Apply filter and play the sound",...
    "constraints", c,...
    "callback", "applyPlay(piece, rate, Filter)",...
    "tag", "applyPlay");

// Apply and play tenth button
c = createConstraints("gridbag", [1 4 1 1], [1/3 1/4], "both", "upper");
applyPlayButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "string", "Apply filter and play a tenth of the sound",...
    "constraints", c,...
    "callback", "applyPlayTenth(piece, rate, Filter)",...
    "tag", "applyPlayTenth");

// Play button
c = createConstraints("gridbag", [2 3 1 1], [1/3 1/4], "both", "upper");
playButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "string", "Play the original sound",...
    "constraints", c,...
    "callback", "play(piece, rate)",...
    "tag", "play");

// Play tenth button
c = createConstraints("gridbag", [2 4 1 1], [1/3 1/4], "both", "upper");
playButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "string", "Play a tenth of the sound (if too long)",...
    "constraints", c,...
    "callback", "playTenth(piece, rate)",...
    "tag", "play");

// Visualize filter button
c = createConstraints("gridbag", [3 3 1 1], [1/3 1/4], "both", "upper");
showFilterButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "string", "Visualize filter",...
    "constraints", c,...
    "callback", "visuFilter(Filter)",...
    "tag", "visuFilter");

// Compare ffts button
c = createConstraints("gridbag", [3 4 1 1], [1/3 1/4], "both", "upper");
showButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "string", "Compare original and new FFTs",...
    "constraints", c,...
    "callback", "visuFFT(piece, Filter)",...
    "tag", "visuFFT");

function [piece, rate] = loadFile()
    [filename, path] = uigetfile("*.*", pwd(), "Select the sound file (.wav)");

    if filename <> "" then
        if isfile(path+filesep()+filename) then
            [path, fname, extension] = fileparts(path+filesep()+filename);
            if extension <> ".wav" then
                messagebox("Wrong extension: WAV file expected", "Wrong extension", "error");
                updateStatusBar(" Erroneous file");
            else

                // File exists and is a .wav one
                [piece, rate] = wavread(path+filesep()+filename);
                if ismatrix(piece)
                    // Stereo sound, only keep an average of the two signals
                    if size(piece, "r") == 2 then
                        piece = (piece(1, :) + piece(2, :))/2;
                    else
                        piece = (piece(:, 1) + piece(:, 2))/2;
                    end
                    updateStatusBar(" File loaded: "+fname+extension+"   Converted to mono sound");
                else
                    updateStatusBar(" File loaded: "+fname+extension);
                end

            end
        else
            messagebox("Please enter an existing file name", "Wrong file name", "error");
            updateStatusBar(" Erroneous file");
        end
    end
endfunction

function Filter = newFrequency()
    freqString = get(gcbo, "string");
    frequency = evstr(freqString);
    if frequency > freqUpperLimit then
        frequency = freqUpperLimit;
        set(gcbo, "string", string(freqUpperLimit));
    elseif frequency < freqLowerLimit then
        frequency = freqLowerLimit;
        set(gcbo, "string", string(freqLowerLimit));
    end

    highButton = get("highRadio");
    if highButton.value == 1 then
        Filter = iir(9, "hp", "butt", frequency/rate, []); // High-pass
    else
        Filter = iir(9, "lp", "butt", frequency/rate, []); // Low-pass
    end
endfunction

function Filter = highPass()
    freqEdit = get(freqBox);
    freqString = get(freqEdit, "string");
    frequency = evstr(freqString);
    Filter = iir(9, "hp", "butt", frequency/rate, []);
endfunction

function Filter = lowPass()
    freqEdit = get(freqBox);
    freqString = get(freqEdit, "string");
    frequency = evstr(freqString);
    Filter = iir(9, "lp", "butt", frequency/rate, []);
endfunction

function applyPlay(piece, rate, Filter)
    updateStatusBar(" Computing and playing the sound...");
    newPiece = flts(piece, Filter);
    playsnd(2 * newPiece, rate); // Correct the filtered sound by increasing the volume by 2
    updateStatusBar("");
endfunction

function applyPlayTenth(piece, rate, Filter)
    updateStatusBar(" Computing and playing a tenth of the sound...");
    newPiece = flts(piece, Filter);
    newPieceTenth = newPiece(:, 1:floor(size(newPiece,"c")/10));
    playsnd(2 * newPieceTenth, rate); // Correct the filtered sound by increasing the volume by 2
    updateStatusBar("");
endfunction

function play(piece, rate)
    updateStatusBar(" Playing the original sound...");
    playsnd(piece, rate);
    updateStatusBar("");
endfunction

function playTenth(piece, rate)
    pieceTenth = piece(:, 1:floor(size(piece,"c")/10));
    updateStatusBar(" Playing a tenth of the original sound...");
    playsnd(pieceTenth, rate);
    updateStatusBar("");
endfunction

function visuFilter(Filter)
    [xm, fr] = frmag(Filter, 1000);
    fr = rate*fr;

    // Preparing the window
    freqEdit = get(freqBox);
    freqString = get(freqEdit, "string");
    fig_filter = figure(...
        "figure_name", "IIR Butterworth filter of order 9 with "+freqString+" cutoff frequency",...
        "figure_size", [800 600],...
        "background", 8,...
        "visible", "off");
    scf(fig_filter);

    // Plotting
    plot(fr(1:70), xm(1:70)) // Restricting to the usual piano keyboard [0;1600]Hz
    fig_filter.visible = "on";
endfunction

function visuFFT(piece, Filter)
    updateStatusBar(" Computing FFTs, please wait...");
    oldFFT = abs(fft(piece));
    newPiece = flts(piece, Filter);
    newFFT = abs(fft(newPiece));
    oldf_v = linspace(0, 1, size(piece, "c")) * rate;
    newf_v = linspace(0, 1, size(newPiece, "c")) * rate;
    // Restricting the spectrum to a usual piano keyboard (up to ~1600Hz)
    id = find(oldf_v < 1600);

    // Preparing the window
    freqEdit = get(freqBox);
    freqString = get(freqEdit, "string");
    fig_ffts = figure(...
        "figure_name", "FFT comparison with "+freqString+" cutoff frequency",...
        "figure_size", [800 600],...
        "background", 8,...
        "visible", "off");
    scf(fig_ffts);

    // Plotting
    subplot(2,1,1);
    plot(oldf_v(id)', oldFFT(id)');
    xtitle("Original sound");
    subplot(2,1,2);
    plot(newf_v(id)', newFFT(id)');
    [xm, fr] = frmag(Filter, 1000);
    fr = rate*fr;
    plot(fr(1:70), xm(1:70)*max(newFFT(id)'), "r--") // Scaling the filter to appear on the plot
    xtitle("Filtered sound with scaled filter superimposed");
    fig_ffts.visible = "on";
    updateStatusBar("");
endfunction

function updateStatusBar(msg)
    set("main_figure", "info_message", msg);
endfunction

funcprot(p);

// Display the GUI
fig.visible = "on";
