// Sound source modification (balance)
// Copyright (C) 2015 - Paul Bignier

p = funcprot();
funcprot(0);

// Defining the GUI
fig = figure(...
    "figure_name", "Balance",...
    "figure_size", [400 200],...
    "layout", "gridbag",...
    "background", 8,...
    "menubar_visible", "off",...
    "toolbar_visible", "off",...
    "default_axes", "off",...
    "visible", "off",...
    "tag", "main_figure");

// Frame for the uicontrols
c = createConstraints("gridbag", [1 1 1 3], [1 1], "both", "right");
main_frame = uicontrol(...
    "style", "frame",...
    "constraints", c,...
    "layout", "gridbag",...
    "tag", "main_frame");

// Text area
c = createConstraints("gridbag", [1 1 1 1], [1 1/3], "both", "upper");
text = uicontrol(...
    "parent", main_frame,...
    "style", "text",...
    "string", "Slide to balance the sound left and right (stereo)",...
    "horizontalAlignment", "center",...
    "constraints", c,...
    "tag", "text");

// Slider
c = createConstraints("gridbag", [1 2 1 1], [1 1/3], "both", "upper");
slider = uicontrol(...
    "parent", main_frame,...
    "style", "slider",...
    "min", -4,...
    "max", 4,...
    "value", 0,...
    "sliderStep", [1 1],...
    "constraints", c,...
    "callback", "y = slider_update(x)",...
    "tag", "slider");

// Play button
c = createConstraints("gridbag", [1 3 1 1], [1 1/3], "both", "upper");
playButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "string", "Play sound",...
    "constraints", c,...
    "callback", "play(y,rate)",...
    "tag", "play");

function y = slider_update(x)
    sl = get("slider");
    value = get(sl, "value");
    ratio = value/4 + 1;
    y = x;
    y(1, :) = (-ratio+2)*y(1, :);
    y(2, :) = ratio*y(2, :);
endfunction

function play(y, rate)
    updateStatusBar(" Playing the sound...");
    sl = get("slider");
    set(sl, "enable", "off");
    playsnd(y, rate);
    set(sl, "enable", "on");
    updateStatusBar("");
endfunction

function updateStatusBar(msg)
    set("main_figure", "info_message", msg);
endfunction

funcprot(p);

// Load the sound from the builtin file
[x, rate] = wavread("SCI/modules/sound/demos/chimes.wav");
y = x;

// Display the GUI and play the sound one first time
fig.visible = "on";
play(y, rate);
